pub fn left_numeric_buffer<A: Iterator<Item = PiecewiseLinearCurvePoint> + Clone, B: Iterator<Item = PiecewiseLinearCurvePoint> + Clone>(
state: NumericBufferState<A>,
) -> NumericBufferState<Either<A, B>>Expand description
Wraps a NumericBufferState<A> as a NumericBufferState<Either<A, B>>
by placing the inner iterator on the Left side.
Useful when you need to return a single NumericBufferState type
but the underlying buffer may come from one of two differently-typed sources.