pub fn right_numeric_buffer<A: Iterator<Item = PiecewiseLinearCurvePoint> + Clone, B: Iterator<Item = PiecewiseLinearCurvePoint> + Clone>(
state: NumericBufferState<B>,
) -> NumericBufferState<Either<A, B>>Expand description
Wraps a NumericBufferState<B> as a NumericBufferState<Either<A, B>>
by placing the inner iterator on the Right side.
Useful when you need to return a single NumericBufferState type
but the underlying buffer may come from one of two differently-typed sources.