SynthStatesMap

Struct SynthStatesMap 

pub struct SynthStatesMap { /* private fields */ }
A simple implementation of SynthParamStates that is backed by a HashMap.

This is useful for testing or other places when you want to pass a SynthParamStates to a component outside of a Conformal wrapper.

Implementations

Source

impl SynthStatesMap

pub fn new_override_defaults<'a, S: AsRef<str> + 'a>( infos: impl IntoIterator<Item = InfoRef<'a, S>> + 'a, overrides: &HashMap<&str, InternalValue>, numeric_expression_overrides: &HashMap<NumericGlobalExpression, f32>, switch_expression_overrides: &HashMap<SwitchGlobalExpression, bool>, ) -> Self

Create a new SynthStatesMap to pass to a synth from a list of Infos and overrides.

This is similar to StatesMap::new_override_defaults, but it also includes expression controllers.

Examples
let infos = vec![
  StaticInfoRef {
    title: "Numeric",
    short_title: "Numeric",
    unique_id: "numeric",
    flags: Default::default(),
    type_specific: TypeSpecificInfoRef::Numeric {
      default: 0.0,
      valid_range: 0.0..=1.0,
      units: None,
    },
  },
];

let overrides = vec![
  // You can override declared parameters
  ("numeric", InternalValue::Numeric(0.5)),
].into_iter().collect();
let numeric_expression_overrides = vec![
  // Or you can override control parameters
  (NumericGlobalExpression::ModWheel, 0.2),
].into_iter().collect();
let states = SynthStatesMap::new_override_defaults(infos.iter().cloned(), &overrides, &numeric_expression_overrides, &Default::default());

// Overridden parameters get the values you passed in
assert_eq!(states.get_numeric("numeric"), Some(0.5));
assert_eq!(states.get_numeric_global_expression(NumericGlobalExpression::ModWheel), 0.2);

// Other parameters get their default values
assert_eq!(states.get_numeric_global_expression(NumericGlobalExpression::PitchBend), 0.0);
pub fn new_with_per_note<'a, S: AsRef<str> + 'a>( infos: impl IntoIterator<Item = InfoRef<'a, S>> + 'a, overrides: &HashMap<&str, InternalValue>, numeric_expression_overrides: &HashMap<NumericGlobalExpression, f32>, switch_expression_overrides: &HashMap<SwitchGlobalExpression, bool>, per_note_expression_overrides: &HashMap<(NumericPerNoteExpression, NoteID), f32>, ) -> Self

Create a new SynthStatesMap with per-note expression overrides.

This is similar to Self::new_override_defaults, but also allows specifying per-note expression values for specific notes.

Examples
let infos = vec![
  StaticInfoRef {
    title: "Numeric",
    short_title: "Numeric",
    unique_id: "numeric",
    flags: Default::default(),
    type_specific: TypeSpecificInfoRef::Numeric {
      default: 0.0,
      valid_range: 0.0..=1.0,
      units: None,
    },
  },
];

let note_id = NoteID::from_pitch(60);
let per_note_overrides = vec![
  ((NumericPerNoteExpression::PitchBend, note_id), 1.5),
].into_iter().collect();

let states = SynthStatesMap::new_with_per_note(
  infos.iter().cloned(),
  &Default::default(),
  &Default::default(),
  &Default::default(),
  &per_note_overrides,
);

assert_eq!(states.get_numeric_expression_for_note(NumericPerNoteExpression::PitchBend, note_id), 1.5);
pub fn new_defaults<'a, S: AsRef<str> + 'a>( infos: impl IntoIterator<Item = InfoRef<'a, S>> + 'a, ) -> Self

Create a new SynthStatesMap to pass to a synth from a list of Infos.

Each parameter in Infos will be set to its default value.

This is similar to StatesMap::new_defaults, but it also includes expression controllers.

Examples
let infos = vec![
  StaticInfoRef {
    title: "Numeric",
    short_title: "Numeric",
    unique_id: "numeric",
    flags: Default::default(),
    type_specific: TypeSpecificInfoRef::Numeric {
      default: 0.0,
      valid_range: 0.0..=1.0,
      units: None,
    },
  },
];

let states = SynthStatesMap::new_defaults(infos.iter().cloned());
assert_eq!(states.get_numeric("numeric"), Some(0.0));

// Controller parameters will also be included
assert_eq!(states.get_numeric_global_expression(NumericGlobalExpression::ModWheel), 0.0);